@@ -5,18 +5,21 @@ import collections
 import itertools
 import logging
 import re
-from collections import namedtuple
+from collections import defaultdict, namedtuple
 
+import six
 from django.core.exceptions import ImproperlyConfigured
+from django.db.models import FileField, Transform
 from django.db.models.aggregates import Count
 from django.db.models.base import ModelBase
 from django.db.models.expressions import Col, RawSQL
+from django.db.models.fields.related_lookups import RelatedExact, RelatedIn
 from django.db.models.lookups import Exact, In, IsNull, Lookup, Range
 from django.db.models.query import EmptyResultSet
 from django.db.models.sql.compiler import SQLCompiler as BaseSQLCompiler
 from django.db.models.sql.constants import CURSOR, MULTI, NO_RESULTS, ORDER_DIR, SINGLE
 from django.db.models.sql.datastructures import BaseTable
-from django.db.models.sql.where import SubqueryConstraint, WhereNode
+from django.db.models.sql.where import NothingNode, SubqueryConstraint, WhereNode
 from django.db.utils import NotSupportedError, OperationalError, ProgrammingError
 
 from rest_models.backend.connexion import build_url
@@ -234,6 +237,7 @@ class QueryParser(object):
         """
         self.query = query
         self._aliases = None
+        self._resolved_ids = None
         """
         the dict to store aliases giving the tuple
         :type: dict[str, Alias]
@@ -331,6 +335,11 @@ class QueryParser(object):
         elif isinstance(col, Col):
             current = self.aliases[col.alias]  # type: Alias
             field = col.target.column
+        elif isinstance(col, Transform):
+            # transform should be passed as is to rest-framework
+            current, transforms = self.resolve_path(col.lhs)
+            transform_name = getattr(col, 'key_name', None) or getattr(col, 'lookup_name')
+            field = '%s.%s' % (transforms, transform_name)
         else:
             raise NotSupportedError("Only Col in sql select is supported")
         if current.m2m is not None:
@@ -387,6 +396,8 @@ class QueryParser(object):
         a query on the api to fetch the ids.
         :return: the list of ids found, or None if it was not possible to find out.
         """
+        if self._resolved_ids is not None:
+            return self._resolved_ids
         ids = None
         first_connector = None
         try:
@@ -404,7 +415,11 @@ class QueryParser(object):
             if negated or not lookup.rhs_is_direct_value() or first_connector != is_and:
                 return None
             # check this lookup is only for main model's primary key
-            if lookup.lhs.alias != main_alias or lookup.lhs.target != self.query.get_meta().pk:
+            try:
+                if lookup.lhs.alias != main_alias or lookup.lhs.target != self.query.get_meta().pk:
+                    return None
+            except AttributeError:
+                # lookup.lhs is not exact or don't have alias. it may be a Transform or something similar.
                 return None
             if isinstance(lookup, Exact):
                 to_add = {lookup.rhs}
@@ -422,6 +437,7 @@ class QueryParser(object):
                 ids |= to_add
         if not ids:
             return None
+        self._resolved_ids = ids
         return ids
 
 
@@ -494,11 +510,12 @@ def join_aliases(aliases, responsereader, existing_aliases):
                 yield subresult
 
 
-def join_results(row, resolved):
+def join_results(row, resolved, connection=None):
     """
     a generator that will generate each results possible for the row data and the resolved data
     :param row:
     :param resolved:
+    :param connection: the connection which can release a cursor for further query (like fetching files on the api)
     :return:
     """
     if not resolved:
@@ -523,13 +540,14 @@ def join_results(row, resolved):
         else:
             field = alias.model._meta.get_field(db_column)
 
-        if hasattr(field, "to_python"):
+        if isinstance(field, FileField) and hasattr(field.storage, 'prepare_result_from_api'):
+            res.append(field.storage.prepare_result_from_api(raw_val, connection.cursor()))
+        elif hasattr(field, "to_python"):
             python_val = field.to_python(raw_val)
             res.append(python_val)
-
         elif isinstance(raw_val, list):
             for val in raw_val:
-                for subresult in join_results(row, resolved[:]):
+                for subresult in join_results(row, resolved[:], connection):
                     yield res + [val] + subresult
             return
         else:
@@ -541,13 +559,13 @@ def join_results(row, resolved):
 
 def simple_count(compiler, result):
     """
-    special case that check if the query is a count on one column and shall return only one resulti.
+    special case that check if the query is a count on one column and shall return only one result.
     this can be made by the pagination hack
     :param SQLCompiler compiler: the compiler that is used
     :param result: the result type
     :return:
     """
-    if len(compiler.select) == 1 and isinstance(compiler.select[0][0], Count) and result is SINGLE:
+    if result is SINGLE and isinstance(compiler.select[-1][0], Count) and result is SINGLE:
         url = get_resource_path(compiler.query.model)
         params = compiler.build_filter_params()
         params['per_page'] = 1
@@ -557,14 +575,71 @@ def simple_count(compiler, result):
             params=params
         )
         compiler.raise_on_response(url, params, response)
-        return True, [response.json()['meta']['total_results']]
+        return True, ([] * (len(compiler.select) - 1)) + [response.json()['meta']['total_results']]
 
     return False, None
 
 
+def introspect_many_to_many_relations(through):
+    """
+    helper to introspect a through model and return all usefull data like the many2many fields,
+    the fk and the related models.
+    return a list of 2 items: one for each «side» of the many2many.
+    each items contains a n3 tuple: the through.foreignkey, the related model, and the ManytoMany(Rel].
+    """
+    fields = []
+    for fk in through._meta.get_fields():
+        if fk.is_relation and fk.many_to_one:
+
+            for rel_model_f in fk.related_model._meta.get_fields():
+                if rel_model_f.is_relation \
+                        and rel_model_f.many_to_many:
+                    model_through = getattr(rel_model_f.remote_field, 'through', None) \
+                                    or getattr(rel_model_f, 'through')
+                    if model_through == through:
+                        fields.append((fk, fk.related_model, rel_model_f))
+                        break
+            else:
+                raise AssertionError("we can't find the field used as relation using through %s" % through)
+
+    return fields
+
+
+def m2m_through(compiler, result):
+    # this si dark magic: instead of quering a auto created table, we get one side of the m2m, and we request it
+    # to get all values in the requested m2m
+    meta = compiler.query.get_meta()
+    if hasattr(meta, 'APIMeta') or not meta.auto_created or not meta.auto_created.APIMeta:
+        # this is a declared api model
+        return False, []
+    # this is a throug model apimodel <-> apimodel
+    assert compiler.query.where.connector == 'AND'  # we can only support a specific query to emuliate query on through
+    a, b = compiler.query.where.children
+    if isinstance(a, RelatedIn) and isinstance(b, RelatedExact):
+        rex = b
+    elif isinstance(b, RelatedIn) and isinstance(a, RelatedExact):
+        rex = a
+    else:
+        raise AssertionError("query on throuhg not supported: %s AND %s" % (a, b))
+    for fk, rel_model, rel_m2m in introspect_many_to_many_relations(meta.model):
+        if rex.lhs.target.related_model == rel_model:
+            response = compiler.connection.cursor().get(
+                get_resource_path(rel_model, rex.rhs),
+                params={'exclude[]': '*', 'include[]': rel_m2m.name}
+            )
+            pks = response.json()[get_resource_name(rel_model)][rel_m2m.name]
+            if result == MULTI:
+                return True, [[(pk,) for pk in pks]]  # chunk of row of cols
+            else:
+                return True, [(pk,) for pk in pks]
+    else:
+        raise AssertionError("did not found a many2many relation using throug=%s" % (meta.model,))
+
+
 class SQLCompiler(BaseSQLCompiler):
     SPECIAL_CASES = [
-        simple_count
+        simple_count,
+        m2m_through,
     ]
 
     META_NAME = 'meta'
@@ -597,7 +672,7 @@ class SQLCompiler(BaseSQLCompiler):
         return RestModelRouter.is_api_model(self.query.model)
 
     def compile(self, node, select_format=False):
-        return None, None
+        return '', []
 
     def check_compatibility(self):
         """
@@ -609,7 +684,7 @@ class SQLCompiler(BaseSQLCompiler):
         query = self.query
         if query.group_by is not None:
             raise FakeDatabaseDbAPI2.NotSupportedError('group by is not supported')
-        if query.distinct and self.connection.settings_dict.get('PREVENT_DISTINCT', True):
+        if query.distinct and self.connection.settings_dict.get('PREVENT_DISTINCT', False):
             raise FakeDatabaseDbAPI2.NotSupportedError('distinct is not supported')
         # check where
         where_nodes = [query.where]
@@ -636,6 +711,8 @@ class SQLCompiler(BaseSQLCompiler):
                         raise FakeDatabaseDbAPI2.NotSupportedError(
                             "nested queryset is not supported"
                         )
+                    elif isinstance(child, NothingNode):
+                        raise EmptyResultSet
                     else:  # pragma: no cover
                         raise ProgrammingError("unknown type for compiling the query : %s."
                                                " expeced a Lookup or WhereNode" % child.__class__)
@@ -654,6 +731,7 @@ class SQLCompiler(BaseSQLCompiler):
         """
         res = {}
         query = self.query
+        list_of_in = {}
         for negated, _, lookup in self.query_parser.flaten_where_clause(query.where):  # type: bool, Lookup
             negated_mark = "-" if negated else ""
             field = self.query_parser.get_rest_path_for_col(lookup.lhs)
@@ -664,12 +742,17 @@ class SQLCompiler(BaseSQLCompiler):
             key = 'filter{%s%s}' % (negated_mark, fieldname)
             if isinstance(lookup.rhs, (tuple, list)):
                 res.setdefault(key, []).extend(lookup.rhs)
+                if lookup.lookup_name == 'in':
+                    list_of_in[key] = list_of_in.get(key, 0) + len(lookup.rhs)
             else:
                 if lookup.lookup_name == 'exact' and res.get(key, lookup.rhs) != lookup.rhs:
                     # a small performance that won't trigger any query if the
                     # queryset ask for differents exacts values
                     raise EmptyResultSet()
                 res.setdefault(key, []).append(lookup.rhs)
+        if not all(list_of_in.values()):
+            # if there is one «xxx.in=[]», me should not query the database
+            raise EmptyResultSet()
         return res
 
     def build_include_exclude_params(self):
@@ -821,6 +904,7 @@ class SQLCompiler(BaseSQLCompiler):
         :return:
         """
         return json.get(self.META_NAME)
+
     # #####################################
     #       real query for select
     # #####################################
@@ -860,7 +944,7 @@ class SQLCompiler(BaseSQLCompiler):
             self.query_parser.resolve_path(col)
             for col, _, _ in self.select
             if not isinstance(col, RawSQL) or col.sql != '1'  # skip special case with exists()
-            ]
+        ]
         if not resolved:
             # nothing in select. special case in exists()
             yield [[]]
@@ -870,7 +954,7 @@ class SQLCompiler(BaseSQLCompiler):
             alias_list = list(resolve_tree(alias_tree))
 
             for row in join_aliases(alias_list, responsereader, {alias_tree.alias: item}):
-                for subresult in join_results(row, resolved):
+                for subresult in join_results(row, resolved, self.connection):
                     yield subresult
 
     def result_iter(self, responsereader):
@@ -896,10 +980,10 @@ class SQLCompiler(BaseSQLCompiler):
         return False, None
 
     def execute_sql(self, result_type=MULTI, chunked_fetch=False, chunk_size=None):
-        self.setup_query()
-        if not result_type:
-            result_type = NO_RESULTS
         try:
+            self.setup_query()
+            if not result_type:
+                result_type = NO_RESULTS
             is_special, result = self.special_cases(result_type)
             if is_special:
                 return result
@@ -956,32 +1040,108 @@ class SQLCompiler(BaseSQLCompiler):
 
 
 class SQLInsertCompiler(SQLCompiler):
+    def resolve_data_n_files(self, obj):
+        """
+        build the dict of files and data to send, from the values of obj.
+
+        :return:
+        """
+
+        data = {}
+        files = {}
+        query = self.query
+        for field in query.fields:
+            if isinstance(field, FileField):
+                fieldfile = getattr(obj, field.attname) if query.raw else field.pre_save(obj, True)
+                # file.name is the return of our storage.save => we get the content file instead of his name
+                # to retreive it there.
+                file = fieldfile.name
+
+                if file is not None:  # field value can be None....
+                    files[field.column] = (file.name, file, file.content_type)
+                else:
+                    data[field.column] = None
+            else:
+                fieldname = field.column
+                data[fieldname] = field.get_db_prep_save(
+                    getattr(obj, field.attname) if query.raw else field.pre_save(obj, True),
+                    connection=self.connection
+                )
+        if files:
+            return data, files
+        else:
+            return data, None
+
+    def handle_insert_through(self):
+        """
+        special case where we don't insert into the given table because it's not exposed. we
+        handle this case by updating the field on either side of the through.
+        :return:
+        """
+        objs = self.query.objs
+        kept_n = None
+        kept_fks = None
+        other_fk = None
+        for fk, rel_model, rel_m2m in introspect_many_to_many_relations(self.query.get_meta().model):
+            cur_n = len({getattr(o, fk.name) for o in objs})
+            if kept_n is None or kept_n > cur_n:
+                other_fk = other_fk or fk
+                kept_n = cur_n
+                kept_fks = fk, rel_model, rel_m2m
+            else:
+                other_fk = fk
+        fk, rel_model, rel_m2m = kept_fks  # we restore the data about the model we will query and update
+
+        vals = defaultdict(list)
+        for obj in objs:
+            vals[getattr(obj, fk.attname)].append(getattr(obj, other_fk.attname))
+
+        for obj_pk, new_vals in vals.items():
+            url = get_resource_path(rel_model, obj_pk)
+            response = self.connection.cursor().get(
+                url,
+                params={'exclude[]': '*', 'include[]': rel_m2m.name}
+            )
+            if response.status_code != 200:
+                raise FakeDatabaseDbAPI2.ProgrammingError(
+                    "error while solving m2m final value at %s [%d]\n%s" %
+                    (url, response.status_code, response.text)
+                )
+            final_vals = set(new_vals) | set(response.json()[get_resource_name(rel_model)][rel_m2m.name])
+            response_update = self.connection.cursor().patch(
+                url,
+                json={get_resource_name(rel_model): {rel_m2m.name: list(final_vals)}}
+            )
+            if response_update.status_code not in (200, 202, 204):
+                raise FakeDatabaseDbAPI2.ProgrammingError(
+                    "error while solving m2m final value at %s [%d]\n%s" %
+                    (url, response_update.status_code, response.text)
+                )
+
     def execute_sql(self, return_id=False, chunk_size=None):
         query = self.query
         """:type: django.db.models.sql.subqueries.InsertQuery"""
         opts = query.get_meta()
+        if not hasattr(opts, 'APIMeta') and opts.auto_created and opts.auto_created.APIMeta:
+            return self.handle_insert_through()
         can_bulk = not return_id and self.connection.features.has_bulk_insert
 
         query_objs = query.objs
-        if can_bulk:
-            # bulk insert
+
+        if can_bulk and not any(f for f in query.fields if isinstance(f, FileField)):
+            # bulk insert if we can and there is no filefield
+            # we send the json data as a dict.
             data = [
-                {
-                    f.column: f.get_db_prep_save(
-                        getattr(obj, f.attname) if query.raw else f.pre_save(obj, True),
-                        connection=self.connection
-                    )
-                    for f in query.fields
-                    }
+                self.resolve_data_n_files(obj)[0]
                 for obj in query_objs
-                ]
+            ]
 
             json = {
                 get_resource_name(query.model, many=True): data
             }
             response = self.connection.cursor().post(
                 get_resource_path(self.query.model),
-                json=json
+                json=json  # send json since we send a dict.
             )
             if response.status_code != 201:
                 raise FakeDatabaseDbAPI2.ProgrammingError(
@@ -993,79 +1153,212 @@ class SQLInsertCompiler(SQLCompiler):
                 for field in opts.concrete_fields:
                     setattr(old, field.attname, field.to_python(new[field.concrete and field.db_column or field.name]))
         else:
+            # we send one object. we send all data as form-encoded data
             result_json = None
             for obj in query_objs:
-                data = {
-                    f.column: f.get_db_prep_save(
-                        getattr(obj, f.attname) if query.raw else f.pre_save(obj, True),
-                        connection=self.connection
+                obj_data, files = self.resolve_data_n_files(obj)
+                if files:
+                    # we make it in 2 requests: first we upload data and files to create the instance
+                    # then we update the instance with json which can be more accurate about None and special values
+
+                    response_files = self.connection.cursor().post(
+                        url=get_resource_path(query.model),
+                        data=obj_data,  # data will not be accurate enouth
+                        files=files
                     )
-                    for f in query.fields
-                    }
+                    if response_files.status_code != 201:
+                        raise FakeDatabaseDbAPI2.ProgrammingError(
+                            "error while creating (uploading files and data) %s with data=%s ; files=%s.\n%s" % (
+                                obj, obj_data, files, message_from_response(response_files)))
+                    # update with json formated
+                    new_id = response_files.json()[get_resource_name(query.model, many=False)]['id']
+                    response = self.connection.cursor().patch(
+                        url=get_resource_path(query.model, pk=new_id),
+                        json={get_resource_name(query.model, many=False): obj_data}
+                    )
+                    if response.status_code not in (200, 202, 204):
+                        raise FakeDatabaseDbAPI2.ProgrammingError("error while updating %s with json=%s.\n%s" % (
+                            obj, obj_data, message_from_response(response)))
+                else:
+                    # update with json formated
+                    response = self.connection.cursor().post(
+                        url=get_resource_path(query.model),
+                        json={get_resource_name(query.model, many=False): obj_data}
+                    )
+                    if response.status_code != 201:
+                        raise FakeDatabaseDbAPI2.ProgrammingError("error while creating %s with json=%s.\n%s" % (
+                            obj, obj_data, message_from_response(response)))
 
-                json = {
-                    get_resource_name(query.model, many=False): data
-                }
-                response = self.connection.cursor().post(
-                    get_resource_path(self.query.model),
-                    json=json
-                )
-                if response.status_code != 201:
-                    raise FakeDatabaseDbAPI2.ProgrammingError("error while creating %s with %s.\n%s" % (
-                        obj, json, message_from_response(response)))
                 result_json = response.json()
                 new = result_json[get_resource_name(query.model, many=False)]
+
                 for field in opts.concrete_fields:
-                    setattr(obj, field.attname, field.to_python(new[field.concrete and field.db_column or field.name]))
+                    try:
+                        raw_val = new[field.concrete and field.db_column or field.name]
+                    except KeyError:
+                        continue
+                    if isinstance(field, FileField) and hasattr(field.storage, 'prepare_result_from_api'):
+                        python_val = field.storage.prepare_result_from_api(raw_val, self.connection.cursor())
+                    elif hasattr(field, "to_python"):
+                        python_val = field.to_python(raw_val)
+                    else:
+                        python_val = raw_val
+                    setattr(obj, field.attname, python_val)
 
             if return_id and result_json:
                 return result_json[get_resource_name(query.model, many=False)][opts.pk.column]
 
 
+class FakeCursor(object):
+    """
+    a fake cursor with rowcount
+    """
+    def __init__(self, rowcount):
+        self.rowcount = rowcount
+
+
 class SQLDeleteCompiler(SQLCompiler):
     def execute_sql(self, result_type=MULTI, chunk_size=None):
+        opts = self.query.get_meta()
         if self.is_api_model():
 
             q = self.query
+            counter = 0
             # we don't care about many2many table, the api will clean it for us
             if not self.query.get_meta().auto_created:
 
-                for id in self.resolve_ids():
+                for id_ in self.resolve_ids():
+                    counter += 1
                     result = self.connection.cursor().delete(
-                        get_resource_path(q.model, pk=id),
+                        get_resource_path(q.model, pk=id_),
                     )
                     if result.status_code not in (200, 202, 204):
                         raise ProgrammingError("the deletion has failed : %s" % result.text)
+            count = counter
+        elif opts.auto_created and opts.auto_created.APIMeta:
+            # through
+            count = self.handle_delete_through()
+        if result_type == CURSOR:
+            return FakeCursor(count)
+
+    def handle_delete_through(self):
+        """
+        special case where we don't insert into the given table because it's not exposed. we
+        handle this case by updating the field on either side of the through.
+        :return:
+        """
+        meta = self.query.get_meta()
+        # this is a throug model apimodel <-> apimodel
+        assert self.query.where.connector == 'AND'  # we can only support a specific query to emuliate query on through
+
+        rin = rex = None
+        for part in self.query.where.children:
+            if isinstance(part, RelatedIn):
+                rin = part
+            elif isinstance(part, RelatedExact):
+                rex = part
+            else:
+                raise AssertionError("unexpected a part of the where clause to be %s for a through update" % part)
+        if rex is None:
+            # special case where we emulate CASCADE on through
+            return 1
+        for fk, rel_model, rel_m2m in introspect_many_to_many_relations(meta.model):
+            if rex.lhs.target.related_model == rel_model:
+                if rin is None:
+                    # clear all entry
+                    final_pks = []
+                else:
+                    response = self.connection.cursor().get(
+                        get_resource_path(rel_model, rex.rhs),
+                        params={'exclude[]': '*', 'include[]': rel_m2m.name}
+                    )
+                    pks = response.json()[get_resource_name(rel_model)][rel_m2m.name]
+                    final_pks = set(pks) - set(rin.rhs)
+                response = self.connection.cursor().patch(
+                    get_resource_path(rel_model, rex.rhs),
+                    json={get_resource_name(rel_model): {rel_m2m.name: list(final_pks)}}
+                )
+                if response.status_code not in (200, 202, 204):
+                    raise ProgrammingError("the deletion of m2m intry has failed : %s" % response.text)
+                return 1
 
 
 class SQLUpdateCompiler(SQLCompiler):
-    def resolve_data(self):
+    def resolve_data_n_files(self):
         """
         build the dict to send with a put/patch that contains the data to update
 
         :return:
         """
-        return {
-            get_resource_name(self.query.model, many=False): {
-                field.concrete and field.db_column or field.name: field.get_db_prep_save(
-                    val, connection=self.connection
-                )
-                for field, _, val in self.query.values
-                }
-        }
+
+        data = {}
+        files = {}
+        query = self.query
+        for field, _, val in query.values:
+            if isinstance(field, FileField):
+                fieldfile = field.pre_save(val.instance, False)
+                # file.name is the return of our storage.save => we get the content file instead of his name
+                # file.name can be a string, in this case, we did got the file at all, he is unmodified
+                file = fieldfile.name
+                if file is None:
+                    # field can be set to None
+                    data[field.column] = None
+                elif isinstance(file, six.string_types):
+                    # str => we don't change it since it's not comming from our custom storage
+                    pass
+                else:
+                    files[field.column] = (file.name, file, file.content_type)
+
+            else:
+                fieldname = field.concrete and field.db_column or field.name
+                data[fieldname] = field.get_db_prep_save(val, connection=self.connection)
+
+        return data, (files or None)
 
     def execute_sql(self, result_type=MULTI, chunk_size=None):
         updated = 0
         if self.is_api_model():
-            q = self.query
-            json = self.resolve_data()
-            for id in self.resolve_ids():
-                result = self.connection.cursor().patch(
-                    get_resource_path(q.model, pk=id),
-                    json=json
-                )
-                if result.status_code not in (200, 202, 204):
-                    raise ProgrammingError("the update has failed : %s" % result.text)
+            query = self.query
+            data, files = self.resolve_data_n_files()
+            for id_ in self.resolve_ids():
+
+                url = get_resource_path(query.model, pk=id_)
+                result_json = {}
+                mixed = ([dict(json=data)] if data else []) + ([dict(files=files)] if files else [])
+                # do json update first and then patch files after.
+                # we can't do this in one request because wa can't mix json and file in the same patch
+                # and mixing data + files should not work because of typed values
+                for kw in mixed:
+                    result = self.connection.cursor().patch(
+                        url,
+                        **kw
+                    )
+                    if result.status_code not in (200, 202, 204):
+                        raise FakeDatabaseDbAPI2.ProgrammingError(
+                            "error while updating %s.pk=%s with data=%s,files=%s.\n%s" % (
+                             query.model.__name__, id_, data, files, message_from_response(result))
+                        )
+
+                    # update object instance using result data if possible
+                    result_json.update(result.json()[get_resource_name(query.model, many=False)])
+
+                instance_data = {}
+                obj = None
+                for field, _, val in self.query.values:
+                    try:
+                        raw_val = result_json[field.concrete and field.db_column or field.name]
+                    except KeyError:
+                        continue
+                    if isinstance(field, FileField) and hasattr(field.storage, 'prepare_result_from_api'):
+                        python_val = field.storage.prepare_result_from_api(raw_val, self.connection.cursor())
+                        obj = val.instance
+                    elif hasattr(field, "to_python"):
+                        python_val = field.to_python(raw_val)
+                    else:
+                        python_val = raw_val
+                    instance_data[field.attname] = python_val
+                if obj:
+                    obj.__dict__.update(instance_data)
                 updated += 1
         return updated
 
@@ -1073,5 +1366,11 @@ class SQLUpdateCompiler(SQLCompiler):
 class SQLAggregateCompiler(SQLCompiler):
 
     def execute_sql(self, result_type=MULTI, chunk_size=None):
+        self.setup_query()
+        if not result_type:
+            result_type = NO_RESULTS
+        is_special, result = self.special_cases(result_type)
+        if is_special:
+            return result
         raise NotSupportedError("the aggregation for the database %s is not supported : %s" % (
-                                self.connection.alias, self.query))  # pragma: no cover
+            self.connection.alias, self.query))  # pragma: no cover
